<?php

namespace Teknisa\MAN\Services;
    
use Doctrine\ORM;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO\Request;
use \Setforms2\Shared\ORM\DateTime;

use Teknisa\MAN\Services\ConsolidacaoVendasService;

class CancelConsVendasService extends ConsolidacaoVendasService {
    
    protected $dataSourceNames = array(
        'filterVendasCancelEst' => 'vnd07900_cancelconsvendas_s_lstvendas',
        'filterVendasNCancelEst' => 'vnd07900_cancelconsvendas_n_lstvendas',
        'filterNroLancto' => 'vnd07900_cancelconsvendas_nrlanctonf',
        'filterAbaNotaFisc' => 'vnd07900_cancelconsvendas_notafis',
        'filterAbaVendas' => 'vnd07900_cancelconsvendas_vendas',
        'filterAbaItensNota' => 'vnd07900_cancelconsvendas_itens_notafis',
        'filterAbaItensEst' => 'vnd07900_cancelconsvendas_itens_est',
        'filterAbaRetirada' => 'vnd07900_cancelconsvendas_ret',
        'filterAbaItensRet' => 'vnd07900_cancelconsvendas_itens_ret'
    );
    
    public function __construct(ORM\EntityManager $entityManager, $consolidationFileLogService, $environment) {
        parent::__construct($entityManager, $consolidationFileLogService, $environment);
    }
    
    public function getPeriodoContParams($cdfilial, $nrorg) {
        $params = array(
            ':CDFILIAL' => $cdfilial,
            ':NRORG'    => $nrorg
        );
        $paramsReturn = $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_PERIODO_CONT, $params);
        return (count($paramsReturn) != 0) ? $paramsReturn[0] : Array();
    }
    
    public function cancelConsolidation($data, $filterParams, $dataProc) {
        $datainicial = '';
        $datafinal = '';
        $cancelConsolidation = array(
            'status'  => true,
            'data'    => array()
        );
        
        try {
            foreach ($data as $row => $lanctoestq) {
                //if se data de processamento vazia, considera data atual para a data final
                if(!empty($dataProc) || $dataProc == null || $dataProc == ''){
                    $dataProc = date('d/m/Y');
                }
                if ((Utilities\Util::formatDateToInteger($lanctoestq['DTLANCESTQ']) < Utilities\Util::formatDateToInteger($dataProc))){
                    $datainicial = $lanctoestq['DTLANCESTQ'];
                }else{
                    $datainicial = $dataProc;
                }
                    
                if ((Utilities\Util::formatDateToInteger($lanctoestq['DTLANCESTQ']) > Utilities\Util::formatDateToInteger($dataProc))){
                    $datafinal = $lanctoestq['DTLANCESTQ'];
                }else {
                    $datafinal = $dataProc; 
                }       
                // chama função para cancelar consolidação 
                self::cancelPerLanctoEst($data[$row], $cancelConsolidation, $lanctoestq['NRLANCESTQ'], $lanctoestq['NRCTRLMOVEST'], $datainicial, $datafinal, $filterParams);
                // verifica se o cancelamento da consolidação prossegue ou não 
                if (!$cancelConsolidation['status']){
                    break;
                }
            }
            return $cancelConsolidation;
        } catch (\Exception $e) {
           throw $e;
        }
    }
    
    public function cancelPerLanctoEst($row, &$cancelConsolidation, $lanctoestq, $nrctrlmovest, $datainicial, $datafinal, $filterParams){
    // cancela consolidação por lançamento de estoque
        $paramsCancel  = array();
        try {
        // começa transação com o banco
            self::controlTransaction('begin');
        } catch (\Exception $e) {
            throw $e;
        }
        try {
            // deleta retirada de estoque
            self::formatCancelLanctoEst($row, $paramsCancel, $filterParams, $lanctoestq, $nrctrlmovest);
            
            $this->entityManager->getConnection()->executeQuery(self::DELETE_RET_ESTQ, $paramsCancel);
            // update item venda para contabilizar o delete
            $this->entityManager->getConnection()->executeQuery(self::UPDATE_CONS_ITEM_VENDA, $paramsCancel);
            // delete controle movimento de estoque
            $this->entityManager->getConnection()->executeQuery(self::DELETE_CTRLMOVIESTQ, $paramsCancel);
            // deleta itens da retirada
            $this->entityManager->getConnection()->executeQuery(self::DELETE_IT_LANCTO_EST, $paramsCancel);
            // deleta lançamento de estoque
            $this->entityManager->getConnection()->executeQuery(self::DELETE_LANCTO_EST, $paramsCancel);
            // executa procedure atualização de estoque
            self::execProcRecEstoq($filterParams, $lanctoestq, $datainicial, $datafinal);
            // executa commit do cancelamento da consolidação por lançamento de estoque
            if ($cancelConsolidation['status'])
                self::controlTransaction('commit');
        } catch (\Exception $e) {
            self::controlTransaction('rollback');
            throw $e;
        }
    }
    
    public function formatCancelLanctoEst($row, &$paramsCancel, $filterParams, $lanctoestq, $nrctrlmovest) {
    // formato utilizado para paramèstros dos campos no cancelamento de lançamento de estoque
        $paramsCancel = array(
            ':CDFILIAL'       => $row['CDFILIAL'],
            ':NRLANCESTQ'     => $lanctoestq,
            ':NRCTRLMOVEST'   => $nrctrlmovest,
            ':NRORG'          => $filterParams['NRORG'] 
        );
    }
    
    public function findConsolidacoesConcluidas ($row, &$paramsCancel, $filterParams, $lanctoestq, $nrctrlmovest) {
        $paramsCancel = array(
            ':CDFILIAL'       => $row['CDFILIAL'],
            ':NRLANCESTQ'     => $lanctoestq,
            ':NRCTRLMOVEST'   => $nrctrlmovest,
            ':NRORG'          => $filterParams['NRORG'] 
        );
        
        return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_CONSOLIDACAO_PENDENTE, $params);
    }
    
    public function find($params) {

 		return $this->entityManager->getConnection()->fetchAll(self::GET_CONSOLIDACOES, $params); 
    }
    
    const GET_CONSOLIDACOES = "SELECT DISTINCT
                    NULL AS CDLOJA,
                    NULL AS CANCRETEST,
                    NULL AS DTVENDA,
                    :DTVENDAINI AS DTVENDAINI,
                    :DTVENDAFIM AS DTVENDAFIM,
                    NULL AS CDCAIXA,
                    :TODASVND AS TODASVND,
                    :LSTVENDAS AS LSTVENDAS,
                    :TCDCAIXA AS TCDCAIXA,
                    :CDFILIAL AS CDFILIALFILTER,
                    LA.CDFILIAL,
                    LA.NRLANCESTQ,
                    TO_CHAR(LA.DTLANCESTQ,'DD/MM/YYYY') AS DTLANCESTQ,
                    RE.NRRETEST,
                    VE.NRORG,
                    CTRL.NRCTRLMOVEST
                FROM
                    VENDA VE
                    JOIN ITEMVENDA IT ON ( IT.NRORG = VE.NRORG )
                                         AND ( IT.CDFILIAL = VE.CDFILIAL )
                                         AND ( IT.CDCAIXA = VE.CDCAIXA )
                                         AND ( IT.NRSEQVENDA = VE.NRSEQVENDA )
                                         AND ( IT.CDFILIALEST IS NOT NULL )
                                         AND ( IT.NRLANCESTQ IS NOT NULL )
                    JOIN LANCTOESTOQ LA ON ( LA.NRORG = VE.NRORG )
                                           AND ( LA.CDFILIAL = IT.CDFILIALEST )
                                           AND ( LA.NRLANCESTQ = IT.NRLANCESTQ )
                    JOIN RETIRADAESTO RE ON ( RE.NRORG = VE.NRORG )
                                            AND ( RE.CDFILIALLAN = LA.CDFILIAL )
                                            AND ( RE.NRLANCESTQ = LA.NRLANCESTQ )
                    LEFT JOIN CTRLMOVIESTQ CTRL ON ( CTRL.CDFILIAL =:CDFILIAL )
                                                   AND ( CTRL.NRORG = LA.NRORG )
                                                   AND ( CTRL.NRLANESTSAID = LA.NRLANCESTQ )
                WHERE
                    ( VE.NRORG =:NRORG )
                    AND ( VE.DTENTRVENDA BETWEEN :DTVENDAINI AND :DTVENDAFIM )
                    AND ( VE.IDSITUVENDA = 'O' )
                    AND ( VE.CDFILIAL =:CDFILIAL )
                    AND (
                        ( INSTR(:CDCAIXA,VE.CDCAIXA) > 0 )
                        OR (:TCDCAIXA = 'T' )
                    )
                    AND (
                        ( INSTR(:LSTVENDAS,VE.CDFILIAL
                                            || VE.CDCAIXA
                                            || VE.NRSEQVENDA) > 0 )
                        OR (:TODASVND = 'S' )
                    )
                ORDER BY
                    RE.NRRETEST";
    
    const GET_CONSOLIDACAO_PENDENTE = "
        SELECT DISTINCT NULL AS CDLOJA, NULL AS CANCRETEST, NULL AS DTVENDA, :DTVENDAINI AS DTVENDAINI, :DTVENDAFIM AS DTVENDAFIM, NULL AS CDCAIXA, :TODASVND AS TODASVND, :LSTVENDAS AS LSTVENDAS, :TCDCAIXA AS TCDCAIXA, :CDFILIAL AS CDFILIALFILTER, LA.CDFILIAL, LA.NRLANCESTQ, TO_CHAR(LA.DTLANCESTQ, 'DD/MM/YYYY') AS DTLANCESTQ, RE.NRRETEST, VE.NRORG, CTRL.NRCTRLMOVEST FROM VENDA VE JOIN ITEMVENDA IT ON (IT.NRORG = VE.NRORG) AND (IT.CDFILIAL = VE.CDFILIAL) AND (IT.CDCAIXA = VE.CDCAIXA) AND (IT.NRSEQVENDA = VE.NRSEQVENDA) AND (IT.CDFILIALEST IS NOT NULL) AND (IT.NRLANCESTQ IS NOT NULL) JOIN LANCTOESTOQ LA ON (LA.NRORG = VE.NRORG) AND (LA.CDFILIAL = IT.CDFILIALEST) AND (LA.NRLANCESTQ = IT.NRLANCESTQ) JOIN RETIRADAESTO RE ON (RE.NRORG = VE.NRORG) AND (RE.CDFILIALLAN = LA.CDFILIAL) AND (RE.NRLANCESTQ = LA.NRLANCESTQ) LEFT JOIN CTRLMOVIESTQ CTRL ON (CTRL.CDFILIAL = :CDFILIAL) AND (CTRL.NRORG = LA.NRORG) AND (CTRL.NRLANESTSAID = LA.NRLANCESTQ) WHERE (VE.NRORG = :NRORG) AND (VE.DTENTRVENDA BETWEEN :DTVENDAINI AND :DTVENDAFIM) AND (VE.IDSITUVENDA = 'O') AND (VE.CDFILIAL = :CDFILIAL) AND ((INSTR(:CDCAIXA, VE.CDCAIXA) > 0) OR (:TCDCAIXA = 'T')) AND ((INSTR(:LSTVENDAS, VE.CDFILIAL ||VE.CDCAIXA ||VE.NRSEQVENDA) > 0) OR (:TODASVND = 'S')) ORDER BY RE.NRRETEST
    ";
    
  
   const DELETE_RET_ESTQ = "
      DELETE FROM RETIRADAESTO
        WHERE CDFILIALLAN = :CDFILIAL AND
          NRLANCESTQ  = :NRLANCESTQ AND
          NRORG = :NRORG
      ";
  
    const UPDATE_CONS_ITEM_VENDA = "
      UPDATE ITEMVENDA
        SET
          CDFILIALEST = NULL,
          NRLANCESTQ = NULL
        WHERE
          CDFILIALEST = :CDFILIAL AND
          NRLANCESTQ = :NRLANCESTQ AND 
          NRORG = :NRORG
      ";  
    const DELETE_LANCTO_EST = "
        DELETE FROM   LANCTOESTOQ
         WHERE 
          CDFILIAL = :CDFILIAL AND
          NRLANCESTQ = :NRLANCESTQ AND
          NRORG = :NRORG
      ";
    const DELETE_IT_LANCTO_EST = "
        DELETE  FROM ITLANCTOEST WHERE  (NRORG    = :NRORG)
            AND (CDFILIAL   = :CDFILIAL)
            AND (NRLANCESTQ = :NRLANCESTQ)
      ";
      
    const DELETE_CTRLMOVIESTQ = "
        DELETE  FROM CTRLMOVIESTQ WHERE  (NRORG    = :NRORG)
            AND (CDFILIAL   = :CDFILIAL)
            AND (NRCTRLMOVEST = :NRCTRLMOVEST)
      ";
      
       
}